//@line 37 "D:/moz1/mozilla/extensions/xpcomviewer/resources/content/xpcomviewer-commands.js"


var gSupportedCommands = null;



/**
 * XpcomViewer's commands controller
 * 
 * @var nsIController
 */
var gXpcomViewerController =
{
  getCommandStateparams : function(aCmd) {},
  doCommandParams : function(aCmd) {},
  
  /**
   * Returns true if command is supported
   * @param  string   ID of command
   * @return boolean
   */
  supportsCommand : function(aCmd)
  {
    return (aCmd in gSupportedCommands);
  }, // end supportsCommand(aCmd)
  
  
  /**
   * Returns true if command is enabled now
   * @param  string   ID of command
   * @return boolean
   */
  isCommandEnabled : function(aCmd)
  {
    if(this.supportsCommand(aCmd)) {
      return gSupportedCommands[aCmd].isCommandEnabled(aCmd);
    }
    
    Components.utils.reportError("gXpcomViewerController::isCommandEnabled " +
                                 "- Unknown command '" + aCmd + "'!");
    return false;
  }, // end isCommandEnabled(aCmd)
  
  
  /**
   * Execute requested command
   * @param  string   ID of command
   * @return boolean
   */
  doCommand : function(aCmd)
  {
    if(this.supportsCommand(aCmd)) {
      return gSupportedCommands[aCmd].doCommand(aCmd);
    }
    
    Components.utils.reportError("gXpcomViewerController::doCommand " +
                                 "- Unknown command '" + aCmd + "'!");
    return false;
  }, // end doCommand(aCmd)
  
  
  /**
   * Fired when command is executed. (This event is not fired automatically
   * it must be fired by code). We do not use it.
   * @param  nsiEvent
   */
  onEvent : function(aEvent) {}

}; // End of gXpcomViewerController



///////////////////////////////////////////////////////////////////////////////
// Single commands implementation


/**
 * Quit XPCOMViewer command
 */
var gCmdQuit =
{
  isCommandEnabled: function(aCommand) { return true; },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand)
  {
    window.close();
  }
}; // End of gCmdQuit


/**
 * Toggle full screen view command
 */
var gCmdFullScreen = 
{
  isCommandEnabled: function(aCommand) { return true; },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
    window.fullScreen = !window.fullScreen;
    // XXX kObserverSrv.notifyObservers(kObserverSrv, "fullScreenModeChanged", "");
  }
}; // End of gCmdFullScreen


/**
 * Show about dialog
 */
var gCmdShowAbout = 
{
  isCommandEnabled: function(aCommand) { return true; },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
    showAboutDialog();
  }
}; // End of gCmdShowAbout


/**
 * Show home page
 */
var gCmdShowHomePage = 
{
  isCommandEnabled: function(aCommand) { return true; },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
    showHomePage();
  }
}; // End of gCmdShowHomePage


/**
 * Show help window
 */
var gCmdShowHelp = 
{
  isCommandEnabled: function(aCommand) { return true; },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
    showHelp();
  }
}; // End of gCmdShowHelp


/**
 * Show JavaScript console
 */
var gCmdShowJsConsole = 
{
  isCommandEnabled: function(aCommand) { return true; },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
    toJavaScriptConsole();
  }
}; // End of gCmdShowJsConsole


/**
 * Show preferences dialog
 */
var gCmdShowPreferences = 
{
  isCommandEnabled: function(aCommand) { return true; },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
    showPreferences();
  }
}; // End of gCmdShowPreferences


/**
 * Save report
 * (implements 'cmd_saveAsXML', 'cmd_saveAsHTML', 'cmd_saveAsTXT')
 */
var gCmdSaveReportAs = 
{
  isCommandEnabled: function(aCommand) { return true; },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
    var type = new String(aCommand).replace("cmd_saveAs", "").toLowerCase();
    
    // Show filepicker
    var fp = Components.classes["@mozilla.org/filepicker;1"].
        createInstance(Components.interfaces.nsIFilePicker);
    fp.init(window, gStrBundle.getString("saveAsDlgLabel"), 
        Components.interfaces.nsIFilePicker.modeSave);
    
    var res = fp.show();
    var file = null;
    
    if(res == Components.interfaces.nsIFilePicker.returnOK) {
      file = fp.file;
    } else if(res == Components.interfaces.nsIFilePicker.returnReplace) {
      file = fp.file;
    }
    
    // Start document
    var doc = null;
    if(type == "xml")
    {
      doc = Components.classes["@mozilla.org/xml/xml-document;1"].
          createInstance(Components.interfaces.nsIDOMDocument);
      doc = doc.implementation.createDocument("", gCurrentTreeView, null);
    }
    else if(type == "html")
    {
      // XXX Attach stylesheet???
      doc = "<html>\n" +
            "<head>\n" +
            "<title>" + kTreeViewsNames[gCurrentTreeView] + "</title>\n" +
            "</head>\n" +
            "<body>\n" +
            "<h1>" + kTreeViewsNames[gCurrentTreeView] + "</h1>\n" + 
            "<ul>\n";
    }
    else
    {
      doc = kTreeViewsNames[gCurrentTreeView] + "\n\n";
    }
    
    // Fill document
    for(var i=0; i<gXpcomTree.view.rowCount; i++)
    {
      var itemName = gXpcomTree.view.getCellText(i, gXpcomTree.columns.getPrimaryColumn());
//@line 278 "D:/moz1/mozilla/extensions/xpcomviewer/resources/content/xpcomviewer-commands.js"
      var itemId = gXpcomTree.view.getCellText(i, gXpcomTree.columns.getColumnAt(1));
//@line 280 "D:/moz1/mozilla/extensions/xpcomviewer/resources/content/xpcomviewer-commands.js"
      
      switch(type)
      { 
        case "xml":
          var itemElm = doc.createElement("item", "");
          itemElm.setAttribute("name", itemName);
          itemElm.setAttribute("id", itemId);
          doc.documentElement.appendChild(itemElm);
          break;
        
        case "html":
          doc = doc + "<li>" + itemName + " - " + itemId + "</li>\n";
          break;
        
        case "txt":
        default:
          doc = doc + kTreeViewsItemNames[gCurrentTreeView] + ": \"" +
              itemName + "\" \"" + itemId + "\"\n";
          break;
      }
    }
    
    // Finish document
    if(type == "xml")
    {
      // XXX Attach stylesheet???
    }
    else if(type == "html")
    {
      doc = doc + "</body>\n</html>";
    }
    
    // Save to file
    var res1 = (type == "xml") ? writeXmlToFile(file, doc)
                               : writeTextToFile(file, doc);
    
    if(!res1) {
      gStatusbar.update(gStrBundle.getString("saveAsFailedMsg"), "error");
    }
  }
}; // End of gCmdSaveAs


/**
 * Show choosen tree view
 * (implements 'cmd_showClasses', 'cmd_showIfaces', 'cmd_showResults')
 */
var gCmdShowView = 
{
  isCommandEnabled: function(aCommand) { return true; },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
    var view = (new String(aCommand)).replace("cmd_show", "").toLowerCase();
    gXpcomViewer.refreshTree(view);
  }
}; // End of gCmdShowView


/**
 * Apply filter on tree items
 */
var gCmdShowFiltered = 
{
  isCommandEnabled: function(aCommand)
  {
    return (gFilterToolbarTextbox.value !== "");
  },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
	  var currentTreeView = gXpcomTreeViewBackup[gCurrentTreeView];
	  var newTreeView = [];
    var searchString = gFilterToolbarTextbox.value.toLowerCase();
    
    var item = null;
    for(item in currentTreeView) {
      var name = currentTreeView[item].mColumns["nameCol"].mCellText.toLowerCase();
      
      if (name.indexOf(searchString)!=-1) {
        newTreeView.push (currentTreeView[item]);
      }
    }
    
	  gXpcomTreeFilter = true;
    
    switch(gCurrentTreeView) {
      case "classes":
        gXpcomTree.view = new TreeClassesPrototype(newTreeView);
        break;
      
      case "ifaces":
        gXpcomTree.view = new TreeIfacesPrototype(newTreeView);
        break;
      
      case "results":
        gXpcomTree.view = new TreeResultsPrototype(newTreeView);
        break;
    }
    
    // Update user interface
    gStatusbar.update(gStrBundle.getFormattedString("filterAppliedMsg",
        [newTreeView.length]), "success");
    document.title = gStrBundle.getFormattedString("xpcomviewerTitle1", 
        [kTreeViewsNames[gCurrentTreeView], newTreeView.length]);
  }
}; // End of gCmdShowFiltered


/**
 * Show all tree items (remove filter)
 */
var gCmdShowAll = 
{
  isCommandEnabled: function(aCommand)
  {
    return gXpcomTreeFilter;
  },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
    switch(gCurrentTreeView)
    {
      case "classes":
        gXpcomTree.view = new TreeClassesPrototype(gXpcomTreeViewBackup[gCurrentTreeView]);
        break;
      
      case "ifaces":
        gXpcomTree.view = new TreeIfacesPrototype(gXpcomTreeViewBackup[gCurrentTreeView]);
        break;
      
      case "results":
        gXpcomTree.view = new TreeResultsPrototype(gXpcomTreeViewBackup[gCurrentTreeView]);
        break;
    }
    gXpcomTreeFilter = false;
    
    var textbox = document.getElementById("main-search-textbox");
    if(textbox) textbox.value = "";
    
    document.title = gStrBundle.getFormattedString("xpcomviewerTitle", 
        [kTreeViewsNames[gCurrentTreeView], gXpcomTree.view.rowCount]);
  }
}; // End of gCmdShowAll


/**
 * Show selected interface source
 * (implements 'cmd_ifaceLocalLookup', 'cmd_ifaceLXRLookup')
 */
var gCmdIfaceSourceLookup = 
{
  isCommandEnabled: function(aCommand) {
    return (gXpcomViewer.getSelectedTreeItem().mIndex == -1) ? false : true;
  },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
    var selected = gXpcomViewer.getSelectedTreeItem();
    var res = false;
    
    // XXX Check if selected item name isn't currently displayed in any tab
    //gCurrentlyDisplayedSources
    
    if(aCommand == "cmd_ifaceLocalLookup") {
      res = gXpcomViewer.ifaceLocalLookup();
    } else {
      res = gXpcomViewer.ifaceLXRLookup();
    }
    
    if(!res) {
      gStatusbar.update(gStrBundle.getString("doCommandError"), "error");
      Components.utils.reportError(e);
    }
  }
}; // End of gCmdIfaceSourceLookup


/**
 * Copy name of selected tree item into clipboard
 */
var gCmdCopyNameOfSel = 
{
  isCommandEnabled: function(aCommand) {
    return (gXpcomViewer.getSelectedTreeItem().mIndex == -1) ? false : true;
  },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
    var selected = gXpcomViewer.getSelectedTreeItem();
    
    var res = copyTextToClipboard(selected.mName);
    if(!res) {
      gStatusbar.update(gStrBundle.getString("doCommandError"), "error");
      Components.utils.reportError(e);
    }
  }
}; // End of gCmdCopyNameOfSel


/**
 * Generate interface initialization code 
 * (implements 'cmd_generateCppCode', 'cmd_generateJsCode')
 */
var gCmdGenerateCode = 
{
  isCommandEnabled: function(aCommand) { 
    return (gXpcomViewer.getSelectedTreeItem().mIndex == -1) ? false : true;
  },
  getCommandStateParams: function(aCommand, aParams, aRefCon) {},
  doCommandParams: function(aCommand, aParams, aRefCon) {},
  doCommand: function(aCommand) 
  {
    var selected = gXpcomViewer.getSelectedTreeItem();
    var res = false;
    
    if(aCommand == "cmd_generateCppCode")
    {
      // XXX ...
    }
    else
    {
      alert(selected.mType);
      
      if(selected.mType == "class" || selected.mType == "ifaceConstant" ||
         selected.mType == "result")
      {
        var template = kPrefs.getCharPref("templates." +
              (selected.mType=="ifaceConstant") ? "constant" : selected.mType);
        template.replace("SELECTED_ITEM", selected.mName);
        res = copyTextToClipboard(template);
      }
      else if(selected.mType == "interface" && gCurrentTreeView == "ifaces")
      {
        var template = kPrefs.getCharPref("templates.iface2");
        template.replace("SELECTED_ITEM", selected.mName);
        res = copyTextToClipboard(template);
      }
      else if(selected.mType == "interface" && gCurrentTreeView == "classes")
      {
        // ...
      }
      else if(selected.mType == "property")
      {
        // ...
      }
    }
    
    if(!res) {
      gStatusbar.update(gStrBundle.getString("doCommandError"), "error");
      Components.utils.reportError(e);
    }
  }
}; // End of gCmdGenerateCode


